@@ -3,8 +3,10 @@
 import asyncio
 import json
 import logging
+import sys
 from time import time
 
+import pkg_resources
 import websockets
 
 _LOGGER = logging.getLogger(__name__)
@@ -13,13 +15,18 @@ STATE_STARTING = "starting"
 STATE_RUNNING = "running"
 STATE_STOPPED = "stopped"
 
+try:
+    VERSION = pkg_resources.require("graphql-subscription-manager")[0].version
+except Exception:  # pylint: disable=broad-except
+    VERSION = "dev"
+
 
 class SubscriptionManager:
     """Subscription manager."""
 
     # pylint: disable=too-many-instance-attributes
 
-    def __init__(self, init_payload, url):
+    def __init__(self, init_payload, url, user_agent=None):
         """Create resources for websocket communication."""
         try:
             self.loop = asyncio.get_running_loop()
@@ -33,6 +40,11 @@ class SubscriptionManager:
         self._client_task = None
         self._session_id = 0
         self._init_payload = init_payload
+        if user_agent is not None:
+            self._user_agent = user_agent
+        else:
+            self._user_agent = "Python/{0[0]}.{0[1]}".format(sys.version_info)
+        self._user_agent += f" graphql-subscription-manager/{VERSION}"
 
     def start(self):
         """Start websocket."""
@@ -66,15 +78,12 @@ class SubscriptionManager:
             self.retry()
             return
 
-        _LOGGER.debug("Running")
-        self._state = STATE_RUNNING
         try:
             await self._running_loop()
         except Exception:  # pylint: disable=broad-except
             if self._state == STATE_STOPPED:
                 _LOGGER.debug("Stopped, Connection error", exc_info=True)
             else:
-                _LOGGER.error("Connection error", exc_info=True)
                 self.retry()
 
     async def stop(self):
@@ -82,10 +91,18 @@ class SubscriptionManager:
         _LOGGER.debug("Stopping client.")
         self._cancel_retry_timer()
 
+        start_time = time()
+
         for subscription_id in self.subscriptions.copy().keys():
             _LOGGER.debug("Sending unsubscribe: %s", subscription_id)
             await self.unsubscribe(subscription_id)
 
+        while (
+            self.websocket is not None
+            and self.subscriptions
+            and (time() - start_time) < 5.0
+        ):
+            await asyncio.sleep(0.1)
         self._state = STATE_STOPPED
         await self._close_websocket()
 
@@ -103,13 +120,14 @@ class SubscriptionManager:
 
     async def subscribe(self, sub_query, callback, timeout=3):
         """Add a new subscription."""
-        current_session_id = self._session_id
+        current_session_id = str(self._session_id)
         self._session_id += 1
         subscription = {
-            "query": sub_query,
-            "type": "subscription_start",
+            "payload": {"query": sub_query},
+            "type": "subscribe",
             "id": current_session_id,
         }
+
         json_subscription = json.dumps(subscription)
         self.subscriptions[current_session_id] = (callback, sub_query)
 
@@ -130,10 +148,8 @@ class SubscriptionManager:
             _LOGGER.warning("Websocket is closed.")
             return
         await self.websocket.send(
-            json.dumps({"id": subscription_id, "type": "subscription_end"})
+            json.dumps({"id": str(subscription_id), "type": "complete"})
         )
-        if self.subscriptions and subscription_id in self.subscriptions:
-            self.subscriptions.pop(subscription_id)
 
     async def _close_websocket(self):
         if self.websocket is None:
@@ -147,8 +163,9 @@ class SubscriptionManager:
         """Process received msg."""
         result = json.loads(msg)
 
-        if (msg_type := result.get("type", "")) == "init_fail":
-            _LOGGER.debug(msg_type)
+        if (msg_type := result.get("type", "")) == "connection_ack":
+            _LOGGER.debug("Running")
+            self._state = STATE_RUNNING
             return
 
         if (subscription_id := result.get("id")) is None:
@@ -156,6 +173,8 @@ class SubscriptionManager:
 
         if msg_type == "complete":
             _LOGGER.debug("Unsubscribe %s successfully.", subscription_id)
+            if self.subscriptions and subscription_id in self.subscriptions:
+                self.subscriptions.pop(subscription_id)
             return
 
         if (data := result.get("payload")) is None:
@@ -187,15 +206,15 @@ class SubscriptionManager:
         self.websocket = await asyncio.wait_for(
             websockets.connect(
                 self._url,
-                subprotocols=["graphql-subscriptions"],
-
+                subprotocols=["graphql-transport-ws"],
+                extra_headers={"User-Agent": self._user_agent},
             ),
             timeout=10,
         )
         await self.websocket.send(
             json.dumps(
                 {
-                    "type": "init",
+                    "type": "connection_init",
                     "payload": self._init_payload,
                 }
             )
@@ -203,7 +222,10 @@ class SubscriptionManager:
 
     async def _running_loop(self):
         k = 0
-        while self.is_running:
+        while self._state in (
+            STATE_RUNNING,
+            STATE_STARTING,
+        ):
             try:
                 msg = await asyncio.wait_for(self.websocket.recv(), timeout=60)
             except asyncio.TimeoutError:
